/* Copyright 2013 - 2020 Patrick Ulbrich <zulu99@gmx.net>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
* MA 02110-1301, USA.
*/

.mailnag-counter {
    background-color: rgba(70, 144, 217, 0.75);
    color: #ffffff;
    border-radius: 24px;
    font-size: 6pt;
    min-width: 14px;
    min-height: 14px;
}

.menu-item-box {
	background-color: rgba(255, 255, 255, 0.05);
	padding:6px;
	border-radius: 4px;
	spacing: 12px;
	width: 24em;
}

.menu-item-box:hover,
.menu-item-box:focus {
	background-color: rgba(255, 255, 255, 0.12);
}

.account-group-label {
	color: #8e8e80;
	font-size: medium;
	font-weight: bold;
}

.more-label {
	color: #8e8e80;
	font-size: small;
	font-style: italic;
}

.sender-label {
	font-size: medium;
	font-weight: bold;
}

.generic-box {
	spacing: 12px;
}

.subject-label {
	font-size: small;
}

.mailnag-date-label {
	color: #8e8e80;
	font-size: small;
}

.overflow-badge {
	color: #393f3f;
	background-color: #59594f; /*#8e8e80;*/
	font-size: small;
	padding: 0em 0.4em 0em 0.4em;
	border-radius: 4px;
}

.avatar {
	border-radius: 4px;
	background-size:contain;
}
